import { Jsonrpc } from '../../libs';
export class OpenWalletNodeProvider {
    constructor(ethereum, netId, supportsSubscriptions) {
        this.subscriptions = {};
        this.subscribers = {};
        this.connected = true;
        this.ethereum = ethereum;
        this.netId = netId;
        this._supportsSubscriptions = supportsSubscriptions;
    }
    supportsSubscriptions() {
        return this._supportsSubscriptions;
    }
    emit(type, message) {
        const subscribers = this.subscribers[type];
        if (subscribers) {
            for (const subs of subscribers) {
                subs(message);
            }
        }
    }
    send(payload, callback) {
        let promise;
        if (payload.method.endsWith('_subscribe')) {
            const request = Object.assign({ networkId: this.netId }, payload);
            promise = this.ethereum.Node.subscribe(request)
                .then(subscription => {
                const subscriptionId = subscription.response;
                this.subscriptions[subscriptionId] = subscription;
                subscription.on('message', message => {
                    this.emit('data', message);
                });
                subscription.on('unsubscribed', () => {
                    delete this.subscriptions[subscriptionId];
                });
                return subscription.response;
            });
        }
        else if (payload.method.endsWith('_unsubscribe')) {
            const subscription = this.subscriptions[payload.params[0]];
            if (subscription) {
                promise = subscription.unsubscribe({
                    networkId: this.netId,
                    method: 'eth_unsubscribe',
                    params: [subscription.response]
                });
            }
            else {
                promise = Promise.reject({ type: 'METHOD_NOT_FOUND', message: "method not found: " + payload.method, code: -32000 });
            }
        }
        else {
            const request = Object.assign({ networkId: this.netId }, payload);
            promise = this.ethereum.Node.send(request);
        }
        const id = typeof payload.id === 'number'
            ? payload.id
            : typeof payload.id === 'string' ? parseInt(payload.id, 10) : Jsonrpc.messageId++;
        promise
            .then(result => callback(null, { id, jsonrpc: payload.jsonrpc, result }))
            .catch(error => callback(null, { id, jsonrpc: payload.jsonrpc, error }));
    }
    on(type, callback) {
        const subs = this.subscribers[type] || [];
        subs.push(callback);
        this.subscribers[type] = subs;
    }
    removeListener(type, callback) {
        const subs = this.subscribers[type] || [];
        const index = subs.indexOf(callback);
        if (index >= 0) {
            subs.splice(index, 1);
        }
        this.subscribers[type] = subs;
    }
    reset() {
        this.subscribers = {};
        for (const id in this.subscriptions) {
            const subs = this.subscriptions[id];
            subs.unsubscribe({
                networkId: this.netId,
                method: 'eth_unsubscribe',
                params: [subs.response]
            });
        }
        this.subscriptions = {};
    }
    static async create(ethereum, netId) {
        if (!await ethereum.Node.canSend()) {
            throw new Error('Node API is not supported');
        }
        const supportedNetworks = await ethereum.Node.supportedNetworks();
        if (supportedNetworks.indexOf(netId) < 0) {
            throw new Error('Network is not supported: ' + netId);
        }
        const supportsSubscriptions = await ethereum.Node.canSubscribe();
        return new OpenWalletNodeProvider(ethereum, netId, supportsSubscriptions);
    }
}
